@@ -1,8 +1,8 @@
 /* aci.c - routines to parse and check acl's */
-/* $OpenLDAP: pkg/ldap/servers/slapd/aci.c,v 1.14.2.11 2009/01/22 00:00:59 kurt Exp $ */
+/* $OpenLDAP$ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 1998-2009 The OpenLDAP Foundation.
+ * Copyright 1998-2022 The OpenLDAP Foundation.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -39,6 +39,7 @@
 #include "slap.h"
 #include "lber_pvt.h"
 #include "lutil.h"
+#include "slap-config.h"
 
 /* use most appropriate size */
 #define ACI_BUF_SIZE 			1024
@@ -281,20 +282,19 @@ aci_list_get_attr_rights(
 			Debug( LDAP_DEBUG_ACL,
 				"        <= aci_list_get_attr_rights "
 				"test %s for %s -> failed\n",
-				bv.bv_val, attr->bv_val, 0 );
+				bv.bv_val, attr->bv_val );
 			continue;
 		}
 
 		Debug( LDAP_DEBUG_ACL,
 			"        <= aci_list_get_attr_rights "
 			"test %s for %s -> ok\n",
-			bv.bv_val, attr->bv_val, 0 );
+			bv.bv_val, attr->bv_val );
 
 		if ( acl_get_part( list, i, ';', &bv ) < 0 ) {
 			Debug( LDAP_DEBUG_ACL,
 				"        <= aci_list_get_attr_rights "
-				"test no rights\n",
-				0, 0, 0 );
+				"test no rights\n" );
 			continue;
 		}
 
@@ -302,7 +302,7 @@ aci_list_get_attr_rights(
 		Debug( LDAP_DEBUG_ACL,
 			"        <= aci_list_get_attr_rights "
 			"rights %s to mask 0x%x\n",
-			bv.bv_val, mask, 0 );
+			bv.bv_val, mask );
 	}
 
 	return mask;
@@ -730,7 +730,7 @@ aci_init( void )
 	rc = register_at( aci_at.desc, aci_at.ad, 0 );
 	if ( rc != LDAP_SUCCESS ) {
 		Debug( LDAP_DEBUG_ANY,
-			"aci_init: at_register failed\n", 0, 0, 0 );
+			"aci_init: at_register failed\n" );
 		return rc;
 	}
 
@@ -742,8 +742,7 @@ aci_init( void )
 
 static int
 dynacl_aci_parse(
-	const char *fname,
-	int lineno,
+	ConfigArgs *c,
 	const char *opts,
 	slap_style_t sty,
 	const char *right,
@@ -753,17 +752,19 @@ dynacl_aci_parse(
 	const char		*text = NULL;
 
 	if ( sty != ACL_STYLE_REGEX && sty != ACL_STYLE_BASE ) {
-		fprintf( stderr, "%s: line %d: "
-			"inappropriate style \"%s\" in \"aci\" by clause\n",
-			fname, lineno, style_strings[sty] );
+		snprintf( c->cr_msg, sizeof( c->cr_msg ),
+			"inappropriate style \"%s\" in \"aci\" by clause",
+			style_strings[sty] );
+		Debug( LDAP_DEBUG_ANY, "%s: %s.\n", c->log, c->cr_msg );
 		return -1;
 	}
 
 	if ( right != NULL && *right != '\0' ) {
 		if ( slap_str2ad( right, &ad, &text ) != LDAP_SUCCESS ) {
-			fprintf( stderr,
-				"%s: line %d: aci \"%s\": %s\n",
-				fname, lineno, right, text );
+			snprintf( c->cr_msg, sizeof( c->cr_msg ),
+				"aci \"%s\": %s",
+				right, text );
+			Debug( LDAP_DEBUG_ANY, "%s: %s.\n", c->log, c->cr_msg );
 			return -1;
 		}
 
@@ -772,10 +773,10 @@ dynacl_aci_parse(
 	}
 
 	if ( !is_at_syntax( ad->ad_type, SLAPD_ACI_SYNTAX) ) {
-		fprintf( stderr, "%s: line %d: "
-			"aci \"%s\": inappropriate syntax: %s\n",
-			fname, lineno, right,
-			ad->ad_type->sat_syntax_oid );
+		snprintf( c->cr_msg, sizeof( c->cr_msg ),
+			"aci \"%s\": inappropriate syntax: %s",
+			right, ad->ad_type->sat_syntax_oid );
+		Debug( LDAP_DEBUG_ANY, "%s: %s.\n", c->log, c->cr_msg );
 		return -1;
 	}
 
@@ -850,7 +851,7 @@ dynacl_aci_mask(
 		
 		Debug( LDAP_DEBUG_ACL, "        <= aci_mask grant %s deny %s\n",
 			  accessmask2str( tgrant, accessmaskbuf, 1 ), 
-			  accessmask2str( tdeny, accessmaskbuf1, 1 ), 0 );
+			  accessmask2str( tdeny, accessmaskbuf1, 1 ) );
 	}
 
 	/* If the entry level aci didn't contain anything valid for the 
@@ -888,7 +889,7 @@ dynacl_aci_mask(
 				op->o_ndn = op->o_bd->be_rootndn;
 			}
 
-			Debug( LDAP_DEBUG_ACL, "        checking ACI of \"%s\"\n", parent_ndn.bv_val, 0, 0 );
+			Debug( LDAP_DEBUG_ACL, "        checking ACI of \"%s\"\n", parent_ndn.bv_val );
 			ret = backend_attribute( op, NULL, &parent_ndn, ad, &bvals, ACL_AUTH );
 
 			if ( !BER_BVISNULL( &op->o_bd->be_rootndn ) ) {
@@ -921,7 +922,7 @@ dynacl_aci_mask(
 					}
 					Debug( LDAP_DEBUG_ACL, "<= aci_mask grant %s deny %s\n", 
 						accessmask2str( tgrant, accessmaskbuf, 1 ),
-						accessmask2str( tdeny, accessmaskbuf1, 1 ), 0 );
+						accessmask2str( tdeny, accessmaskbuf1, 1 ) );
 				}
 				break;
 
@@ -929,13 +930,13 @@ dynacl_aci_mask(
 				/* just go on if the aci-Attribute is not present in
 				 * the current entry 
 				 */
-				Debug( LDAP_DEBUG_ACL, "no such attribute\n", 0, 0, 0 );
+				Debug( LDAP_DEBUG_ACL, "no such attribute\n" );
 				stop = 0;
 				break;
 
 			case LDAP_NO_SUCH_OBJECT:
 				/* We have reached the base object */
-				Debug( LDAP_DEBUG_ACL, "no such object\n", 0, 0, 0 );
+				Debug( LDAP_DEBUG_ACL, "no such object\n" );
 				stop = 1;
 				break;
 
@@ -1060,7 +1061,7 @@ OpenLDAPaciValidatePerms(
 			break;
 
 		default:
-		        Debug( LDAP_DEBUG_ACL, "aciValidatePerms: perms needs to be one of x,d,c,s,r,w in '%s'\n", perms->bv_val, 0, 0 );
+		        Debug( LDAP_DEBUG_ACL, "aciValidatePerms: perms needs to be one of x,d,c,s,r,w in '%s'\n", perms->bv_val );
 			return LDAP_INVALID_SYNTAX;
 		}
 
@@ -1074,7 +1075,7 @@ OpenLDAPaciValidatePerms(
 		assert( i != perms->bv_len );
 
 		if ( perms->bv_val[ i ] != ',' ) {
-		        Debug( LDAP_DEBUG_ACL, "aciValidatePerms: missing comma in '%s'\n", perms->bv_val, 0, 0 );
+		        Debug( LDAP_DEBUG_ACL, "aciValidatePerms: missing comma in '%s'\n", perms->bv_val );
 			return LDAP_INVALID_SYNTAX;
 		}
 
@@ -1103,7 +1104,7 @@ OpenLDAPaciValidateRight(
 	if ( acl_get_part( action, 0, ';', &bv ) < 0 ||
 		bv_getcaseidx( &bv, ACIgrantdeny ) == -1 )
 	{
-		Debug( LDAP_DEBUG_ACL, "aciValidateRight: '%s' must be either 'grant' or 'deny'\n", bv.bv_val, 0, 0 );
+		Debug( LDAP_DEBUG_ACL, "aciValidateRight: '%s' must be either 'grant' or 'deny'\n", bv.bv_val );
 		return LDAP_INVALID_SYNTAX;
 	}
 
@@ -1137,13 +1138,13 @@ OpenLDAPaciValidateRight(
 				{
 					if ( slap_bv2ad( &attr, &ad, &text ) != LDAP_SUCCESS ) 
 					{
-						Debug( LDAP_DEBUG_ACL, "aciValidateRight: unknown attribute: '%s'\n", attr.bv_val, 0, 0 );
+						Debug( LDAP_DEBUG_ACL, "aciValidateRight: unknown attribute: '%s'\n", attr.bv_val );
 						return LDAP_INVALID_SYNTAX;
 					}
 				} else {
 					if ( slap_bv2ad( &left, &ad, &text ) != LDAP_SUCCESS ) 
 					{
-						Debug( LDAP_DEBUG_ACL, "aciValidateRight: unknown attribute: '%s'\n", left.bv_val, 0, 0 );
+						Debug( LDAP_DEBUG_ACL, "aciValidateRight: unknown attribute: '%s'\n", left.bv_val );
 						return LDAP_INVALID_SYNTAX;
 					}
 				}
@@ -1156,7 +1157,7 @@ OpenLDAPaciValidateRight(
 		return LDAP_SUCCESS;
 
 	} else {
-		Debug( LDAP_DEBUG_ACL, "aciValidateRight: perms:attr need to be pairs in '%s'\n", action->bv_val, 0, 0 );
+		Debug( LDAP_DEBUG_ACL, "aciValidateRight: perms:attr need to be pairs in '%s'\n", action->bv_val );
 		return LDAP_INVALID_SYNTAX;
 	}
 
@@ -1177,12 +1178,12 @@ OpenLDAPaciNormalizeRight(
 
 	/* grant|deny */
 	if ( acl_get_part( action, 0, ';', &grantdeny ) < 0 ) {
-	        Debug( LDAP_DEBUG_ACL, "aciNormalizeRight: missing ';' in '%s'\n", action->bv_val, 0, 0 );
+	        Debug( LDAP_DEBUG_ACL, "aciNormalizeRight: missing ';' in '%s'\n", action->bv_val );
 		return LDAP_INVALID_SYNTAX;
 	}
 	idx = bv_getcaseidx( &grantdeny, ACIgrantdeny );
 	if ( idx == -1 ) {
-	        Debug( LDAP_DEBUG_ACL, "aciNormalizeRight: '%s' must be grant or deny\n", grantdeny.bv_val, 0, 0 );
+	        Debug( LDAP_DEBUG_ACL, "aciNormalizeRight: '%s' must be grant or deny\n", grantdeny.bv_val );
 		return LDAP_INVALID_SYNTAX;
 	}
 
@@ -1220,17 +1221,17 @@ OpenLDAPaciNormalizeRight(
 				{
 					ad = NULL;
 					text = NULL;
-					/* openldap 2.1 aci compabitibility [entry] -> entry */
+					/* openldap 2.1 aci compatibility [entry] -> entry */
 					if ( ber_bvstrcasecmp( &attr, &aci_bv[ ACI_BV_BR_ENTRY ] ) == 0 ) {
 						ad = &adstatic;
 						adstatic.ad_cname = aci_bv[ ACI_BV_ENTRY ];
 
-					/* openldap 2.1 aci compabitibility [children] -> children */
+					/* openldap 2.1 aci compatibility [children] -> children */
 					} else if ( ber_bvstrcasecmp( &attr, &aci_bv[ ACI_BV_BR_CHILDREN ] ) == 0 ) {
 						ad = &adstatic;
 						adstatic.ad_cname = aci_bv[ ACI_BV_CHILDREN ];
 
-					/* openldap 2.1 aci compabitibility [all] -> only [all] */
+					/* openldap 2.1 aci compatibility [all] -> only [all] */
 					} else if ( ber_bvstrcasecmp( &attr, &aci_bv[ ACI_BV_BR_ALL ] ) == 0 ) {
 						ber_memfree_x( nattrs.bv_val, ctx );
 						nattrs = aci_bv[ ACI_BV_BR_ALL ];
@@ -1243,7 +1244,7 @@ OpenLDAPaciNormalizeRight(
 						if ( slap_bv2ad( &attr, &ad, &text ) != LDAP_SUCCESS ) 
 						{
 							ber_memfree_x( nattrs.bv_val, ctx );
-							Debug( LDAP_DEBUG_ACL, "aciNormalizeRight: unknown attribute: '%s'\n", attr.bv_val, 0, 0 );
+							Debug( LDAP_DEBUG_ACL, "aciNormalizeRight: unknown attribute: '%s'\n", attr.bv_val );
 							return LDAP_INVALID_SYNTAX;
 						}
 
@@ -1251,7 +1252,7 @@ OpenLDAPaciNormalizeRight(
 						if ( slap_bv2ad( &left, &ad, &text ) != LDAP_SUCCESS ) 
 						{
 							ber_memfree_x( nattrs.bv_val, ctx );
-							Debug( LDAP_DEBUG_ACL, "aciNormalizeRight: unknown attribute: '%s'\n", left.bv_val, 0, 0 );
+							Debug( LDAP_DEBUG_ACL, "aciNormalizeRight: unknown attribute: '%s'\n", left.bv_val );
 							return LDAP_INVALID_SYNTAX;
 						}
 					}
@@ -1259,7 +1260,7 @@ OpenLDAPaciNormalizeRight(
 				
 					len = nattrs.bv_len + ( !BER_BVISEMPTY( &nattrs ) ? STRLENOF( "," ) : 0 )
 				      		+ ad->ad_cname.bv_len;
-					nattrs.bv_val = ber_memrealloc_x( nattrs.bv_val, len + 1, ctx );
+					nattrs.bv_val = slap_sl_realloc( nattrs.bv_val, len + 1, ctx );
 	                        	ptr = &nattrs.bv_val[ nattrs.bv_len ];
 					if ( !BER_BVISEMPTY( &nattrs ) ) {
 						*ptr++ = ',';
@@ -1271,7 +1272,7 @@ OpenLDAPaciNormalizeRight(
 
 			}
 
-			naction->bv_val = ber_memrealloc_x( naction->bv_val,
+			naction->bv_val = slap_sl_realloc( naction->bv_val,
 				naction->bv_len + STRLENOF( ";" )
 				+ perms.bv_len + STRLENOF( ";" )
 				+ nattrs.bv_len + 1,
@@ -1298,7 +1299,7 @@ OpenLDAPaciNormalizeRight(
 		return LDAP_SUCCESS;
 
 	} else {
-		Debug( LDAP_DEBUG_ACL, "aciNormalizeRight: perms:attr need to be pairs in '%s'\n", action->bv_val, 0, 0 );
+		Debug( LDAP_DEBUG_ACL, "aciNormalizeRight: perms:attr need to be pairs in '%s'\n", action->bv_val );
 		return LDAP_INVALID_SYNTAX;
 	}
 }
@@ -1346,7 +1347,7 @@ OpenLDAPaciNormalizeRights(
 			*nactions = nbv;
 
 		} else {
-			nactions->bv_val = ber_memrealloc_x( nactions->bv_val,
+			nactions->bv_val = slap_sl_realloc( nactions->bv_val,
 				nactions->bv_len + STRLENOF( "$" )
 				+ nbv.bv_len + 1,
 				ctx );
@@ -1411,7 +1412,7 @@ OpenLDAPaciValidate(
 	int		rc;
 	
 	if ( BER_BVISEMPTY( val ) ) {
-		Debug( LDAP_DEBUG_ACL, "aciValidatet: value is empty\n", 0, 0, 0 );
+		Debug( LDAP_DEBUG_ACL, "aciValidatet: value is empty\n" );
 		return LDAP_INVALID_SYNTAX;
 	}
 
@@ -1423,7 +1424,7 @@ OpenLDAPaciValidate(
 		 * I'd replace it with X-ORDERED VALUES so that
 		 * it's guaranteed values are maintained and used
 		 * in the desired order */
-		Debug( LDAP_DEBUG_ACL, "aciValidate: invalid oid '%s'\n", oid.bv_val, 0, 0 );
+		Debug( LDAP_DEBUG_ACL, "aciValidate: invalid oid '%s'\n", oid.bv_val );
 		return LDAP_INVALID_SYNTAX;
 	}
 
@@ -1431,7 +1432,7 @@ OpenLDAPaciValidate(
 	if ( acl_get_part( val, 1, '#', &scope ) < 0 || 
 		bv_getcaseidx( &scope, OpenLDAPaciscopes ) == -1 )
 	{
-		Debug( LDAP_DEBUG_ACL, "aciValidate: invalid scope '%s'\n", scope.bv_val, 0, 0 );
+		Debug( LDAP_DEBUG_ACL, "aciValidate: invalid scope '%s'\n", scope.bv_val );
 		return LDAP_INVALID_SYNTAX;
 	}
 
@@ -1444,7 +1445,7 @@ OpenLDAPaciValidate(
 
 	/* type */
 	if ( acl_get_part( val, 3, '#', &type ) < 0 ) {
-		Debug( LDAP_DEBUG_ACL, "aciValidate: missing type in '%s'\n", val->bv_val, 0, 0 );
+		Debug( LDAP_DEBUG_ACL, "aciValidate: missing type in '%s'\n", val->bv_val );
 		return LDAP_INVALID_SYNTAX;
 	}
 	idx = bv_getcaseidx( &type, OpenLDAPacitypes );
@@ -1452,13 +1453,13 @@ OpenLDAPaciValidate(
 		struct berval	isgr;
 
 		if ( acl_get_part( &type, 0, '/', &isgr ) < 0 ) {
-			Debug( LDAP_DEBUG_ACL, "aciValidate: invalid type '%s'\n", type.bv_val, 0, 0 );
+			Debug( LDAP_DEBUG_ACL, "aciValidate: invalid type '%s'\n", type.bv_val );
 			return LDAP_INVALID_SYNTAX;
 		}
 
 		idx = bv_getcaseidx( &isgr, OpenLDAPacitypes );
 		if ( idx == -1 || idx >= LAST_OPTIONAL ) {
-			Debug( LDAP_DEBUG_ACL, "aciValidate: invalid type '%s'\n", isgr.bv_val, 0, 0 );
+			Debug( LDAP_DEBUG_ACL, "aciValidate: invalid type '%s'\n", isgr.bv_val );
 			return LDAP_INVALID_SYNTAX;
 		}
 	}
@@ -1466,7 +1467,7 @@ OpenLDAPaciValidate(
 	/* subject */
 	bv_get_tail( val, &type, &subject );
 	if ( subject.bv_val[ 0 ] != '#' ) {
-		Debug( LDAP_DEBUG_ACL, "aciValidate: missing subject in '%s'\n", val->bv_val, 0, 0 );
+		Debug( LDAP_DEBUG_ACL, "aciValidate: missing subject in '%s'\n", val->bv_val );
 		return LDAP_INVALID_SYNTAX;
 	}
 
@@ -1477,13 +1478,13 @@ OpenLDAPaciValidate(
 
 			rc = slap_bv2ad( &subject, &ad, &text );
 			if ( rc != LDAP_SUCCESS ) {
-				Debug( LDAP_DEBUG_ACL, "aciValidate: unknown dn attribute '%s'\n", subject.bv_val, 0, 0 );
+				Debug( LDAP_DEBUG_ACL, "aciValidate: unknown dn attribute '%s'\n", subject.bv_val );
 				return LDAP_INVALID_SYNTAX;
 			}
 
 			if ( ad->ad_type->sat_syntax != slap_schema.si_syn_distinguishedName ) {
 				/* FIXME: allow nameAndOptionalUID? */
-				Debug( LDAP_DEBUG_ACL, "aciValidate: wrong syntax for dn attribute '%s'\n", subject.bv_val, 0, 0 );
+				Debug( LDAP_DEBUG_ACL, "aciValidate: wrong syntax for dn attribute '%s'\n", subject.bv_val );
 				return LDAP_INVALID_SYNTAX;
 			}
 		}
@@ -1517,13 +1518,13 @@ OpenLDAPaciValidate(
 
 				rc = slap_bv2ad( &atbv, &ad, &text );
 				if ( rc != LDAP_SUCCESS ) {
-				        Debug( LDAP_DEBUG_ACL, "aciValidate: unknown group attribute '%s'\n", atbv.bv_val, 0, 0 );
+				        Debug( LDAP_DEBUG_ACL, "aciValidate: unknown group attribute '%s'\n", atbv.bv_val );
 					return LDAP_INVALID_SYNTAX;
 				}
 			}
 
 			if ( oc_bvfind( &ocbv ) == NULL ) {
-			        Debug( LDAP_DEBUG_ACL, "aciValidate: unknown group '%s'\n", ocbv.bv_val, 0, 0 );
+			        Debug( LDAP_DEBUG_ACL, "aciValidate: unknown group '%s'\n", ocbv.bv_val );
 				return LDAP_INVALID_SYNTAX;
 			}
 		}
@@ -1531,7 +1532,7 @@ OpenLDAPaciValidate(
 
 	if ( BER_BVISEMPTY( &subject ) ) {
 		/* empty DN invalid */
-	        Debug( LDAP_DEBUG_ACL, "aciValidate: missing dn in '%s'\n", val->bv_val, 0, 0 );
+	        Debug( LDAP_DEBUG_ACL, "aciValidate: missing dn in '%s'\n", val->bv_val );
 		return LDAP_INVALID_SYNTAX;
 	}
 
@@ -1541,7 +1542,7 @@ OpenLDAPaciValidate(
 	/* FIXME: pass DN syntax? */
 	rc = dnValidate( NULL, &subject );
 	if ( rc != LDAP_SUCCESS ) {
-	        Debug( LDAP_DEBUG_ACL, "aciValidate: invalid dn '%s'\n", subject.bv_val, 0, 0 );
+	        Debug( LDAP_DEBUG_ACL, "aciValidate: invalid dn '%s'\n", subject.bv_val );
 	}
 	return rc;
 }
@@ -1570,7 +1571,7 @@ OpenLDAPaciPrettyNormal(
 	BER_BVZERO( out );
 
 	if ( BER_BVISEMPTY( val ) ) {
-		Debug( LDAP_DEBUG_ACL, "aciPrettyNormal: value is empty\n", 0, 0, 0 );
+		Debug( LDAP_DEBUG_ACL, "aciPrettyNormal: value is empty\n" );
 		return LDAP_INVALID_SYNTAX;
 	}
 
@@ -1578,25 +1579,25 @@ OpenLDAPaciPrettyNormal(
 	if ( acl_get_part( val, 0, '#', &oid ) < 0 || 
 		numericoidValidate( NULL, &oid ) != LDAP_SUCCESS )
 	{
-		Debug( LDAP_DEBUG_ACL, "aciPrettyNormal: invalid oid '%s'\n", oid.bv_val, 0, 0 );
+		Debug( LDAP_DEBUG_ACL, "aciPrettyNormal: invalid oid '%s'\n", oid.bv_val );
 		return LDAP_INVALID_SYNTAX;
 	}
 
 	/* scope: normalize by replacing with OpenLDAPaciscopes */
 	if ( acl_get_part( val, 1, '#', &scope ) < 0 ) {
-		Debug( LDAP_DEBUG_ACL, "aciPrettyNormal: missing scope in '%s'\n", val->bv_val, 0, 0 );
+		Debug( LDAP_DEBUG_ACL, "aciPrettyNormal: missing scope in '%s'\n", val->bv_val );
 		return LDAP_INVALID_SYNTAX;
 	}
 	idx = bv_getcaseidx( &scope, OpenLDAPaciscopes );
 	if ( idx == -1 ) {
-		Debug( LDAP_DEBUG_ACL, "aciPrettyNormal: invalid scope '%s'\n", scope.bv_val, 0, 0 );
+		Debug( LDAP_DEBUG_ACL, "aciPrettyNormal: invalid scope '%s'\n", scope.bv_val );
 		return LDAP_INVALID_SYNTAX;
 	}
 	scope = *OpenLDAPaciscopes[ idx ];
 
 	/* rights */
 	if ( acl_get_part( val, 2, '#', &rights ) < 0 ) {
-		Debug( LDAP_DEBUG_ACL, "aciPrettyNormal: missing rights in '%s'\n", val->bv_val, 0, 0 );
+		Debug( LDAP_DEBUG_ACL, "aciPrettyNormal: missing rights in '%s'\n", val->bv_val );
 		return LDAP_INVALID_SYNTAX;
 	}
 	if ( OpenLDAPaciNormalizeRights( &rights, &nrights, ctx )
@@ -1607,7 +1608,7 @@ OpenLDAPaciPrettyNormal(
 
 	/* type */
 	if ( acl_get_part( val, 3, '#', &type ) < 0 ) {
-		Debug( LDAP_DEBUG_ACL, "aciPrettyNormal: missing type in '%s'\n", val->bv_val, 0, 0 );
+		Debug( LDAP_DEBUG_ACL, "aciPrettyNormal: missing type in '%s'\n", val->bv_val );
 		rc = LDAP_INVALID_SYNTAX;
 		goto cleanup;
 	}
@@ -1616,14 +1617,14 @@ OpenLDAPaciPrettyNormal(
 		struct berval	isgr;
 
 		if ( acl_get_part( &type, 0, '/', &isgr ) < 0 ) {
-		        Debug( LDAP_DEBUG_ACL, "aciPrettyNormal: invalid type '%s'\n", type.bv_val, 0, 0 );
+		        Debug( LDAP_DEBUG_ACL, "aciPrettyNormal: invalid type '%s'\n", type.bv_val );
 			rc = LDAP_INVALID_SYNTAX;
 			goto cleanup;
 		}
 
 		idx = bv_getcaseidx( &isgr, OpenLDAPacitypes );
 		if ( idx == -1 || idx >= LAST_OPTIONAL ) {
-		        Debug( LDAP_DEBUG_ACL, "aciPrettyNormal: invalid type '%s'\n", isgr.bv_val, 0, 0 );
+		        Debug( LDAP_DEBUG_ACL, "aciPrettyNormal: invalid type '%s'\n", isgr.bv_val );
 			rc = LDAP_INVALID_SYNTAX;
 			goto cleanup;
 		}
@@ -1634,7 +1635,7 @@ OpenLDAPaciPrettyNormal(
 	bv_get_tail( val, &type, &subject );
 
 	if ( BER_BVISEMPTY( &subject ) || subject.bv_val[ 0 ] != '#' ) {
-	        Debug( LDAP_DEBUG_ACL, "aciPrettyNormal: missing subject in '%s'\n", val->bv_val, 0, 0 );
+	        Debug( LDAP_DEBUG_ACL, "aciPrettyNormal: missing subject in '%s'\n", val->bv_val );
 		rc = LDAP_INVALID_SYNTAX;
 		goto cleanup;
 	}
@@ -1655,7 +1656,7 @@ OpenLDAPaciPrettyNormal(
 			freesubject = 1;
 
 		} else {
-	                Debug( LDAP_DEBUG_ACL, "aciPrettyNormal: invalid subject dn '%s'\n", subject.bv_val, 0, 0 );
+	                Debug( LDAP_DEBUG_ACL, "aciPrettyNormal: invalid subject dn '%s'\n", subject.bv_val );
 			goto cleanup;
 		}
 
@@ -1688,7 +1689,7 @@ OpenLDAPaciPrettyNormal(
 	
 					rc = slap_bv2ad( &atbv, &ad, &text );
 					if ( rc != LDAP_SUCCESS ) {
-	                                        Debug( LDAP_DEBUG_ACL, "aciPrettyNormal: unknown group attribute '%s'\n", atbv.bv_val, 0, 0 );
+	                                        Debug( LDAP_DEBUG_ACL, "aciPrettyNormal: unknown group attribute '%s'\n", atbv.bv_val );
 						rc = LDAP_INVALID_SYNTAX;
 						goto cleanup;
 					}
@@ -1698,13 +1699,13 @@ OpenLDAPaciPrettyNormal(
 
 				oc = oc_bvfind( &ocbv );
 				if ( oc == NULL ) {
-                                        Debug( LDAP_DEBUG_ACL, "aciPrettyNormal: invalid group '%s'\n", ocbv.bv_val, 0, 0 );
+                                        Debug( LDAP_DEBUG_ACL, "aciPrettyNormal: invalid group '%s'\n", ocbv.bv_val );
 					rc = LDAP_INVALID_SYNTAX;
 					goto cleanup;
 				}
 
 				bv.bv_len += STRLENOF( "/" ) + oc->soc_cname.bv_len;
-				bv.bv_val = ber_memalloc_x( bv.bv_len + 1, ctx );
+				bv.bv_val = slap_sl_malloc( bv.bv_len + 1, ctx );
 
 				ptr = bv.bv_val;
 				ptr = lutil_strncopy( ptr, ntype.bv_val, ntype.bv_len );
@@ -1734,14 +1735,14 @@ OpenLDAPaciPrettyNormal(
 
 		rc = slap_bv2ad( &subject, &ad, &text );
 		if ( rc != LDAP_SUCCESS ) {
-                        Debug( LDAP_DEBUG_ACL, "aciPrettyNormal: unknown dn attribute '%s'\n", subject.bv_val, 0, 0 );
+                        Debug( LDAP_DEBUG_ACL, "aciPrettyNormal: unknown dn attribute '%s'\n", subject.bv_val );
 			rc = LDAP_INVALID_SYNTAX;
 			goto cleanup;
 		}
 
 		if ( ad->ad_type->sat_syntax != slap_schema.si_syn_distinguishedName ) {
 			/* FIXME: allow nameAndOptionalUID? */
-                        Debug( LDAP_DEBUG_ACL, "aciPrettyNormal: wrong syntax for dn attribute '%s'\n", subject.bv_val, 0, 0 );
+                        Debug( LDAP_DEBUG_ACL, "aciPrettyNormal: wrong syntax for dn attribute '%s'\n", subject.bv_val );
 			rc = LDAP_INVALID_SYNTAX;
 			goto cleanup;
 		}
@@ -1763,7 +1764,7 @@ OpenLDAPaciPrettyNormal(
 		+ ntype.bv_len + STRLENOF( "#" )
 		+ nsubject.bv_len;
 
-	out->bv_val = ber_memalloc_x( out->bv_len + 1, ctx );
+	out->bv_val = slap_sl_malloc( out->bv_len + 1, ctx );
 	ptr = lutil_strncopy( out->bv_val, oid.bv_val, oid.bv_len );
 	ptr[ 0 ] = '#';
 	ptr++;
