@@ -1,8 +1,8 @@
 /* ldapsync.c -- LDAP Content Sync Routines */
-/* $OpenLDAP: pkg/ldap/servers/slapd/ldapsync.c,v 1.32.2.13 2010/04/19 16:53:02 quanah Exp $ */
+/* $OpenLDAP$ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 2003-2010 The OpenLDAP Foundation.
+ * Copyright 2003-2022 The OpenLDAP Foundation.
  * Portions Copyright 2003 IBM Corporation.
  * All rights reserved.
  *
@@ -36,7 +36,8 @@ slap_compose_sync_cookie(
 	struct berval *cookie,
 	BerVarray csn,
 	int rid,
-	int sid )
+	int sid,
+	struct berval *delcsn )
 {
 	int len, numcsn = 0;
 
@@ -65,6 +66,8 @@ slap_compose_sync_cookie(
 		len = 0;
 		for ( i=0; i<numcsn; i++)
 			len += csn[i].bv_len + 1;
+		if ( delcsn && !BER_BVISEMPTY(delcsn) )
+			len += STRLENOF(",delcsn=") + delcsn->bv_len;
 
 		len += STRLENOF("rid=123,csn=");
 		if ( sid >= 0 )
@@ -83,6 +86,10 @@ slap_compose_sync_cookie(
 			*ptr++ = ';';
 		}
 		ptr--;
+		if ( delcsn && !BER_BVISEMPTY(delcsn) ) {
+			ptr = lutil_strcopy( ptr, ",delcsn=" );
+			ptr = lutil_strncopy( ptr, delcsn->bv_val, delcsn->bv_len );
+		}
 		*ptr = '\0';
 		cookie->bv_len = ptr - cookie->bv_val;
 	}
@@ -112,6 +119,11 @@ slap_sync_cookie_free(
 		BER_BVZERO( &cookie->octet_str );
 	}
 
+	if ( !BER_BVISNULL( &cookie->delcsn )) {
+		ch_free( cookie->delcsn.bv_val );
+		BER_BVZERO( &cookie->delcsn );
+	}
+
 	if ( free_cookie ) {
 		ch_free( cookie );
 	}
@@ -166,6 +178,90 @@ slap_parse_csn_sids( BerVarray csns, int numcsns, void *memctx )
 	return ret;
 }
 
+static slap_mr_match_func sidsort_cmp;
+
+static const MatchingRule sidsort_mr = {
+	{ 0 },
+	NULL,
+	{ 0 },
+	{ 0 },
+	0,
+	NULL, NULL, NULL, sidsort_cmp
+};
+static const AttributeType sidsort_at = {
+	{ 0 },
+	{ 0 },
+	NULL, NULL, (MatchingRule *)&sidsort_mr,
+	NULL, NULL, NULL, NULL, NULL, NULL, NULL, SLAP_AT_SORTED_VAL
+};
+static const AttributeDescription sidsort_ad = {
+	NULL,
+	(AttributeType *)&sidsort_at
+};
+
+static int
+sidsort_cmp(
+	int *matchp,
+	slap_mask_t flags,
+	Syntax *syntax,
+	MatchingRule *mr,
+	struct berval *b1,
+	void *v2 )
+{
+	struct berval *b2 = v2;
+	*matchp = b1->bv_len - b2->bv_len;
+	return LDAP_SUCCESS;
+}
+
+/* sort CSNs by SID. Use a fake Attribute with our own
+ * syntax and matching rule, which sorts the nvals by
+ * bv_len order. Stuff our sids into the bv_len.
+ */
+int
+slap_sort_csn_sids( BerVarray csns, int *sids, int numcsns, void *memctx )
+{
+	Attribute a;
+	const char *text;
+	int i, rc;
+
+	a.a_desc = (AttributeDescription *)&sidsort_ad;
+	a.a_nvals = slap_sl_malloc( numcsns * sizeof(struct berval), memctx );
+	for ( i=0; i<numcsns; i++ ) {
+		a.a_nvals[i].bv_len = sids[i];
+		a.a_nvals[i].bv_val = NULL;
+	}
+	a.a_vals = csns;
+	a.a_numvals = numcsns;
+	a.a_flags = 0;
+	rc = slap_sort_vals( (Modifications *)&a, &text, &i, memctx );
+	for ( i=0; i<numcsns; i++ )
+		sids[i] = a.a_nvals[i].bv_len;
+	slap_sl_free( a.a_nvals, memctx );
+	return rc;
+}
+
+void
+slap_insert_csn_sids(
+	struct sync_cookie *ck,
+	int pos,
+	int sid,
+	struct berval *csn
+)
+{
+	int i;
+	ck->numcsns++;
+	ck->ctxcsn = ch_realloc( ck->ctxcsn,
+		(ck->numcsns+1) * sizeof(struct berval));
+	BER_BVZERO( &ck->ctxcsn[ck->numcsns] );
+	ck->sids = ch_realloc( ck->sids, ck->numcsns * sizeof(int));
+	for ( i = ck->numcsns-1; i > pos; i-- ) {
+		ck->ctxcsn[i] = ck->ctxcsn[i-1];
+		ck->sids[i] = ck->sids[i-1];
+	}
+	ck->sids[i] = sid;
+	ber_dupbv( &ck->ctxcsn[i], csn );
+}
+
 int
 slap_parse_sync_cookie(
 	struct sync_cookie *cookie,
@@ -189,6 +285,7 @@ slap_parse_sync_cookie(
 	cookie->ctxcsn = NULL;
 	cookie->sids = NULL;
 	cookie->numcsns = 0;
+	BER_BVZERO( &cookie->delcsn );
 
 	end = cookie->octet_str.bv_val + cookie->octet_str.bv_len;
 
@@ -275,15 +372,76 @@ slap_parse_sync_cookie(
 			}
 			continue;
 		}
+		if ( !strncmp( next, "delcsn=", STRLENOF("delcsn=") )) {
+			struct berval stamp;
+
+			next += STRLENOF("delcsn=");
+			while ( next < end ) {
+				csn_str = next;
+				csn_ptr = strchr( csn_str, '#' );
+				if ( !csn_ptr || csn_ptr > end )
+					break;
+				/* ad will be NULL when called from main. we just
+				 * want to parse the rid then. But we still iterate
+				 * through the string to find the end.
+				 */
+				cval = strchr( csn_ptr, ';' );
+				if ( !cval )
+					cval = strchr(csn_ptr, ',' );
+				if ( cval )
+					stamp.bv_len = cval - csn_str;
+				else
+					stamp.bv_len = end - csn_str;
+				if ( ad ) {
+					struct berval bv;
+					stamp.bv_val = csn_str;
+					if ( ad->ad_type->sat_syntax->ssyn_validate(
+						ad->ad_type->sat_syntax, &stamp ) != LDAP_SUCCESS )
+						break;
+					if ( ad->ad_type->sat_equality->smr_normalize(
+						SLAP_MR_VALUE_OF_ATTRIBUTE_SYNTAX,
+						ad->ad_type->sat_syntax,
+						ad->ad_type->sat_equality,
+						&stamp, &bv, memctx ) != LDAP_SUCCESS )
+						break;
+					cookie->delcsn = bv;
+				}
+				if ( cval ) {
+					next = cval + 1;
+				} else {
+					next = end;
+				}
+				break;
+			}
+			continue;
+		}
 		next++;
 	}
 	if ( cookie->numcsns ) {
 		cookie->sids = slap_parse_csn_sids( cookie->ctxcsn, cookie->numcsns,
 			memctx );
+		if ( cookie->numcsns > 1 )
+			slap_sort_csn_sids( cookie->ctxcsn, cookie->sids, cookie->numcsns, memctx );
 	}
 	return 0;
 }
 
+/* count the numcsns and regenerate the list of SIDs in a recomposed cookie */
+void
+slap_reparse_sync_cookie(
+	struct sync_cookie *cookie,
+	void *memctx )
+{
+	if ( cookie->ctxcsn ) {
+		for (; !BER_BVISNULL( &cookie->ctxcsn[cookie->numcsns] ); cookie->numcsns++);
+	}
+	if ( cookie->numcsns ) {
+		cookie->sids = slap_parse_csn_sids( cookie->ctxcsn, cookie->numcsns, NULL );
+		if ( cookie->numcsns > 1 )
+			slap_sort_csn_sids( cookie->ctxcsn, cookie->sids, cookie->numcsns, memctx );
+	}
+}
+
 int
 slap_init_sync_cookie_ctxcsn(
 	struct sync_cookie *cookie
@@ -309,6 +467,7 @@ slap_init_sync_cookie_ctxcsn(
 	value_add_one( &cookie->ctxcsn, &ctxcsn );
 	cookie->numcsns = 1;
 	cookie->sid = -1;
+	BER_BVZERO( &cookie->delcsn );
 
 	return 0;
 }
@@ -353,6 +512,10 @@ slap_dup_sync_cookie(
 			new->sids[i] = src->sids[i];
 	}
 
+	if ( !BER_BVISNULL( &src->delcsn )) {
+		ber_dupbv( &new->delcsn, &src->delcsn );
+	}
+
 	if ( !BER_BVISNULL( &src->octet_str )) {
 		ber_dupbv( &new->octet_str, &src->octet_str );
 	}
