@@ -2,7 +2,7 @@
 /* $OpenLDAP$ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 2004-2024 The OpenLDAP Foundation.
+ * Copyright 2004-2022 The OpenLDAP Foundation.
  * Portions Copyright 2004 Symas Corporation.
  * All rights reserved.
  *
@@ -603,7 +603,6 @@ refint_repair(
 	op->o_ndn = op->o_bd->be_rootndn;
 	cache = op->o_do_not_cache;
 	op->o_do_not_cache = 1;
-	op->o_abandon = 0;
 
 	/* search */
 	rc = op->o_bd->be_search( op, &rs );
@@ -949,7 +948,6 @@ refint_response(
 	refint_pre *rp;
 	slap_overinst *on;
 	refint_data *id;
-	BerValue pdn;
 	refint_q *rq;
 	refint_attrs *ip;
 	int ac;
@@ -971,18 +969,8 @@ refint_response(
 	rq->do_sub = rp->do_sub;
 
 	if ( op->o_tag == LDAP_REQ_MODRDN ) {
-		if ( op->oq_modrdn.rs_newSup ) {
-			pdn = *op->oq_modrdn.rs_newSup;
-		} else {
-			dnParent( &op->o_req_dn, &pdn );
-		}
-		build_new_dn( &rq->newdn, &pdn, &op->orr_newrdn, NULL );
-		if ( op->oq_modrdn.rs_nnewSup ) {
-			pdn = *op->oq_modrdn.rs_nnewSup;
-		} else {
-			dnParent( &op->o_req_ndn, &pdn );
-		}
-		build_new_dn( &rq->newndn, &pdn, &op->orr_nnewrdn, NULL );
+		ber_dupbv( &rq->newdn, &op->orr_newDN );
+		ber_dupbv( &rq->newndn, &op->orr_nnewDN );
 	}
 
 	ldap_pvt_thread_mutex_lock( &id->qmutex );
