@@ -2,7 +2,7 @@
 /* $OpenLDAP$ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 2003-2024 The OpenLDAP Foundation.
+ * Copyright 2003-2022 The OpenLDAP Foundation.
  * Portions Copyright 2003 Pierangelo Masarati.
  * All rights reserved.
  *
@@ -122,6 +122,12 @@ rwm_op_rollback( Operation *op, SlapReply *rs, rwm_op_state *ros )
 			op->orr_newrdn = ros->orr_newrdn;
 			op->orr_nnewrdn = ros->orr_nnewrdn;
 		}
+		if ( op->orr_newDN.bv_val != ros->orr_newDN.bv_val ) {
+			ch_free( op->orr_newDN.bv_val );
+			ch_free( op->orr_nnewDN.bv_val );
+			op->orr_newDN = ros->orr_newDN;
+			op->orr_nnewDN = ros->orr_nnewDN;
+		}
 		break;
 	case LDAP_REQ_SEARCH:
 		op->o_tmpfree( ros->mapped_attrs, op->o_tmpmemctx );
@@ -724,6 +730,7 @@ rwm_op_modrdn( Operation *op, SlapReply *rs )
 	slap_overinst		*on = (slap_overinst *) op->o_bd->bd_info;
 	struct ldaprwmap	*rwmap = 
 			(struct ldaprwmap *)on->on_bi.bi_private;
+	struct berval pdn, pndn;
 	
 	int			rc;
 	dncookie		dc;
@@ -758,6 +765,8 @@ rwm_op_modrdn( Operation *op, SlapReply *rs )
 			*op->orr_newSup = newSup;
 			*op->orr_nnewSup = nnewSup;
 		}
+		pdn = newSup;
+		pndn = nnewSup;
 	}
 
 	/*
@@ -795,6 +804,16 @@ rwm_op_modrdn( Operation *op, SlapReply *rs )
 		send_ldap_error( op, rs, rc, "renameDN massage error" );
 		goto err;
 	}
+	if ( !op->orr_newSup ) {
+		dnParent( &op->o_req_dn, &pdn );
+		dnParent( &op->o_req_ndn, &pndn );
+	}
+
+	/*
+	 * Update the new DN
+	 */
+	build_new_dn( &op->orr_newDN, &pdn, &op->orr_newrdn, op->o_tmpmemctx );
+	build_new_dn( &op->orr_nnewDN, &pndn, &op->orr_nnewrdn, op->o_tmpmemctx );
 
 	op->o_callback = &roc->cb;
 
@@ -817,6 +836,13 @@ err:;
 			op->orr_newrdn = roc->ros.orr_newrdn;
 			op->orr_nnewrdn = roc->ros.orr_nnewrdn;
 		}
+
+		if ( op->orr_newDN.bv_val != roc->ros.orr_newDN.bv_val ) {
+			op->o_tmpfree( op->orr_newDN.bv_val, op->o_tmpmemctx );
+			op->o_tmpfree( op->orr_nnewDN.bv_val, op->o_tmpmemctx );
+			op->orr_newDN = roc->ros.orr_newDN;
+			op->orr_nnewDN = roc->ros.orr_nnewDN;
+		}
 	}
 
 	return rc;
@@ -867,6 +893,8 @@ rwm_entry_release_rw( Operation *op, Entry *e, int rw )
 	return SLAP_CB_CONTINUE;
 }
 
+static struct berval *passwd_oid;
+
 static int
 rwm_entry_get_rw( Operation *op, struct berval *ndn,
 	ObjectClass *oc, AttributeDescription *at, int rw, Entry **ep )
@@ -884,6 +912,11 @@ rwm_entry_get_rw( Operation *op, struct berval *ndn,
 		return SLAP_CB_CONTINUE;
 	}
 
+	/* If we're fetching the target of a password mod, must let real DNs thru */
+	if ( op->o_tag == LDAP_REQ_EXTENDED && bvmatch( passwd_oid, &op->oq_extended.rs_reqoid ) ) {
+		return SLAP_CB_CONTINUE;
+	}
+
 	/* massage DN */
 	op2.o_tag = LDAP_REQ_SEARCH;
 	op2 = *op;
@@ -1116,6 +1149,8 @@ static struct exop {
 	{ BER_BVNULL, NULL }
 };
 
+static struct berval *passwd_oid = &exop_table[0].oid;
+
 static int
 rwm_extended( Operation *op, SlapReply *rs )
 {
@@ -1780,6 +1815,16 @@ rwm_response( Operation *op, SlapReply *rs )
 		break;
 	}
 
+	if ( op->o_tag == LDAP_REQ_ADD && op->ora_e ) {
+		/*
+		 * Rewrite back the dn and attributes of the added entry op->ora_e
+		 */
+		SlapReply rs2 = *rs;
+		rs2.sr_entry = op->ora_e;
+		rs2.sr_flags |= REP_ENTRY_MODIFIABLE;
+		return rwm_send_entry( op, &rs2 );
+	}
+
 	return SLAP_CB_CONTINUE;
 }
 
