@@ -1,7 +1,7 @@
-/* $OpenLDAP: pkg/ldap/servers/slapd/slapi/slapi_pblock.c,v 1.63.2.11 2010/04/13 20:23:51 kurt Exp $ */
+/* $OpenLDAP$ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 2002-2010 The OpenLDAP Foundation.
+ * Copyright 2002-2022 The OpenLDAP Foundation.
  * Portions Copyright 1997,2002-2003 IBM Corporation.
  * All rights reserved.
  *
@@ -1019,6 +1019,18 @@ pblock_set( Slapi_PBlock *pb, int param, void *value )
 			rc = pblock_set_dn( value, &pb->pb_op->orr_newrdn, &pb->pb_op->orr_nnewrdn, pb->pb_op->o_tmpmemctx );
 			if ( rc == LDAP_SUCCESS )
 				rc = rdn_validate( &pb->pb_op->orr_nnewrdn );
+			if ( rc == LDAP_SUCCESS ) {
+				struct berval pdn, pndn;
+				if ( pb->pb_op->orr_nnewSup ) {
+					pdn = *pb->pb_op->orr_newSup;
+					pndn = *pb->pb_op->orr_nnewSup;
+				} else {
+					dnParent( &pb->pb_op->o_req_dn, &pdn );
+					dnParent( &pb->pb_op->o_req_ndn, &pndn );
+				}
+				build_new_dn( &pb->pb_op->orr_newDN, &pdn, &pb->pb_op->orr_newrdn, pb->pb_op->o_tmpmemctx );
+				build_new_dn( &pb->pb_op->orr_nnewDN, &pndn, &pb->pb_op->orr_nnewrdn, pb->pb_op->o_tmpmemctx );
+			}
 		} else {
 			rc = PBLOCK_ERROR;
 		}
@@ -1028,6 +1040,7 @@ pblock_set( Slapi_PBlock *pb, int param, void *value )
 		PBLOCK_VALIDATE_IS_INTOP( pb );
 		if ( pb->pb_op->o_tag == LDAP_REQ_MODRDN ) {
 			if ( value == NULL ) {
+				struct berval pdn, pndn;
 				if ( pb->pb_op->orr_newSup != NULL ) {
 					pb->pb_op->o_tmpfree( pb->pb_op->orr_newSup, pb->pb_op->o_tmpmemctx );
 					BER_BVZERO( pb->pb_op->orr_newSup );
@@ -1038,6 +1051,10 @@ pblock_set( Slapi_PBlock *pb, int param, void *value )
 					BER_BVZERO( pb->pb_op->orr_nnewSup );
 					pb->pb_op->orr_nnewSup = NULL;
 				}
+				dnParent( &pb->pb_op->o_req_dn, &pdn );
+				build_new_dn( &pb->pb_op->orr_newDN, &pdn, &pb->pb_op->orr_newrdn, pb->pb_op->o_tmpmemctx );
+				dnParent( &pb->pb_op->o_req_ndn, &pndn );
+				build_new_dn( &pb->pb_op->orr_nnewDN, &pndn, &pb->pb_op->orr_nnewrdn, pb->pb_op->o_tmpmemctx );
 			} else {
 				if ( pb->pb_op->orr_newSup == NULL ) {
 					pb->pb_op->orr_newSup = (struct berval *)pb->pb_op->o_tmpalloc(
@@ -1050,6 +1067,10 @@ pblock_set( Slapi_PBlock *pb, int param, void *value )
 					BER_BVZERO( pb->pb_op->orr_nnewSup );
 				}
 				rc = pblock_set_dn( value, pb->pb_op->orr_newSup, pb->pb_op->orr_nnewSup, pb->pb_op->o_tmpmemctx );
+				if ( rc == LDAP_SUCCESS ) {
+					build_new_dn( &pb->pb_op->orr_newDN, pb->pb_op->orr_newSup, &pb->pb_op->orr_newrdn, pb->pb_op->o_tmpmemctx );
+					build_new_dn( &pb->pb_op->orr_nnewDN, pb->pb_op->orr_nnewSup, &pb->pb_op->orr_nnewrdn, pb->pb_op->o_tmpmemctx );
+				}
 			}
 		} else {
 			rc = PBLOCK_ERROR;
@@ -1176,13 +1197,8 @@ pblock_set( Slapi_PBlock *pb, int param, void *value )
 		break;
 	case SLAPI_SEARCH_RESULT_ENTRY:
 		PBLOCK_ASSERT_OP( pb, 0 );
-		if ( pb->pb_rs->sr_flags & REP_ENTRY_MUSTBEFREED ) {
-			entry_free( pb->pb_rs->sr_entry );
-		} else if ( pb->pb_rs->sr_flags & REP_ENTRY_MUSTRELEASE ) {
-			be_entry_release_r( pb->pb_op, pb->pb_rs->sr_entry );
-			pb->pb_rs->sr_flags ^= REP_ENTRY_MUSTRELEASE;
-		}
-		pb->pb_rs->sr_entry = (Slapi_Entry *)value;
+		rs_replace_entry( pb->pb_op, pb->pb_rs, NULL, (Slapi_Entry *)value );
+		/* TODO: Should REP_ENTRY_MODIFIABLE be set? */
 		pb->pb_rs->sr_flags |= REP_ENTRY_MUSTBEFREED;
 		break;
 	case SLAPI_BIND_RET_SASLCREDS:
@@ -1429,4 +1445,3 @@ slapi_int_pblock_get_next( Slapi_PBlock **pb )
 }
 
 #endif /* LDAP_SLAPI */
-
