@@ -1,7 +1,7 @@
-/* $OpenLDAP: pkg/ldap/servers/slapd/slapi/slapi_utils.c,v 1.189.2.15 2010/04/13 20:23:51 kurt Exp $ */
+/* $OpenLDAP$ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 2002-2010 The OpenLDAP Foundation.
+ * Copyright 2002-2022 The OpenLDAP Foundation.
  * Portions Copyright 1997,2002-2003 IBM Corporation.
  * All rights reserved.
  *
@@ -31,7 +31,11 @@
 #include <slap.h>
 #include <slapi.h>
 
+#ifdef _WIN32
+#include <winsock.h>
+#else
 #include <netdb.h>
+#endif
 
 #ifdef LDAP_SLAPI
 
@@ -1843,6 +1847,42 @@ slapi_pw_find(
 	return 1;
 }
 
+/* Get connected client IP address.
+ *
+ * The user must free the returned client IP after its use.
+ * Compatible with IBM Tivoli call.
+ *
+ * Errors:
+ * * LDAP_PARAM_ERROR - If the pb parameter is null.
+ * * LDAP_OPERATIONS_ERROR - If the API encounters error processing the request.
+ * * LDAP_NO_MEMORY - Failed to allocate required memory.
+ */
+int
+slapi_get_client_ip(Slapi_PBlock *pb, char **clientIP)
+{
+	char *s = NULL;
+
+	if(pb == NULL || pb->pb_conn == NULL) return(LDAP_PARAM_ERROR);
+	if((s = (char *) slapi_ch_malloc(pb->pb_conn->c_peer_name.bv_len + 1)) == NULL) {
+		return(LDAP_NO_MEMORY);
+	}
+
+	memcpy(s, pb->pb_conn->c_peer_name.bv_val, pb->pb_conn->c_peer_name.bv_len);
+
+	s[pb->pb_conn->c_peer_name.bv_len] = 0;
+
+	*clientIP = s;
+
+	return(LDAP_SUCCESS);
+}
+
+/* Free previously allocated client IP address. */
+void
+slapi_free_client_ip(char **clientIP)
+{
+	slapi_ch_free((void **) clientIP);
+}
+
 #define MAX_HOSTNAME 512
 
 char *
@@ -1935,6 +1975,8 @@ slapi_timer_current_time( void )
 	 */
 #else /* _WIN32 */
 	LARGE_INTEGER now;
+	static LARGE_INTEGER base_time, performance_freq;
+	static int performance_counter_present;
 
 	if ( first_time ) {
 		first_time = 0;
@@ -2011,7 +2053,7 @@ int slapi_is_connection_ssl( Slapi_PBlock *pb, int *isSSL )
 }
 
 /*
- * DS 5.x compatability API follow
+ * DS 5.x compatibility API follow
  */
 
 int slapi_attr_get_flags( const Slapi_Attr *attr, unsigned long *flags )
@@ -2108,12 +2150,7 @@ int slapi_attr_add_value( Slapi_Attr *a, const Slapi_Value *v )
 		nvalp = NULL;
 	}
 
-	rc = value_add_one( &a->a_vals, (Slapi_Value *)v );
-	if ( rc == 0 && nvalp != NULL ) {
-		rc = value_add_one( &a->a_nvals, nvalp );
-	} else {
-		a->a_nvals = a->a_vals;
-	}
+	rc = attr_valadd( a, (Slapi_Value *)v, nvalp, 1 );
 
 	if ( nvalp != NULL ) {
 		slapi_ch_free_string( &nval.bv_val );
