@@ -1,7 +1,7 @@
-/* $OpenLDAP: pkg/ldap/tests/progs/slapd-tester.c,v 1.46.2.9 2009/01/22 00:01:18 kurt Exp $ */
+/* $OpenLDAP$ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 1999-2009 The OpenLDAP Foundation.
+ * Copyright 1999-2022 The OpenLDAP Foundation.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -40,12 +40,18 @@
 #include "lber_pvt.h"
 #include "slapd-common.h"
 
-#define SEARCHCMD		"slapd-search"
-#define READCMD			"slapd-read"
-#define ADDCMD			"slapd-addel"
-#define MODRDNCMD		"slapd-modrdn"
-#define MODIFYCMD		"slapd-modify"
-#define BINDCMD			"slapd-bind"
+#ifdef _WIN32
+#define EXE		".exe"
+#else
+#define EXE
+#endif
+
+#define SEARCHCMD		"slapd-search" EXE
+#define READCMD			"slapd-read" EXE
+#define ADDCMD			"slapd-addel" EXE
+#define MODRDNCMD		"slapd-modrdn" EXE
+#define MODIFYCMD		"slapd-modify" EXE
+#define BINDCMD			"slapd-bind" EXE
 #define MAXARGS      		100
 #define MAXREQS			5000
 #define LOOPS			100
@@ -86,7 +92,7 @@ usage( char *name, char opt )
 
 	fprintf( stderr,
 		"usage: %s "
-		"-H <uri> | ([-h <host>] -p <port>) "
+		"-H <uri> "
 		"-D <manager> "
 		"-w <passwd> "
 		"-d <datadir> "
@@ -110,8 +116,6 @@ main( int argc, char **argv )
 {
 	int		i, j;
 	char		*uri = NULL;
-	char		*host = "localhost";
-	char		*port = NULL;
 	char		*manager = NULL;
 	char		*passwd = NULL;
 	char		*dirname = NULL;
@@ -139,6 +143,8 @@ main( int argc, char **argv )
 	int		sanum;
 	int		sextra_args = 0;
 	char		scmd[MAXPATHLEN];
+	int		swamp = 0;
+	char		swampopt[sizeof("-SSS")];
 	/* static so that its address can be used in initializer below. */
 	static char	sloops[LDAP_PVT_INTTYPE_CHARS(unsigned long)];
 	/* read */
@@ -209,7 +215,7 @@ main( int argc, char **argv )
 	mloops[0] = '\0';
 	bloops[0] = '\0';
 
-	while ( ( i = getopt( argc, argv, "AB:CD:d:FH:h:Ii:j:L:l:NP:p:r:t:Ww:y:" ) ) != EOF )
+	while ( ( i = getopt( argc, argv, "AB:CD:d:FH:h:Ii:j:L:l:NP:p:r:St:Ww:y:" ) ) != EOF )
 	{
 		switch ( i ) {
 		case 'A':
@@ -243,7 +249,7 @@ main( int argc, char **argv )
 			break;
 
 		case 'd':		/* data directory */
-			dirname = strdup( optarg );
+			dirname = optarg;
 			break;
 
 		case 'F':
@@ -251,11 +257,7 @@ main( int argc, char **argv )
 			break;
 
 		case 'H':		/* slapd uri */
-			uri = strdup( optarg );
-			break;
-
-		case 'h':		/* slapd host */
-			host = strdup( optarg );
+			uri = optarg;
 			break;
 
 		case 'I':
@@ -317,7 +319,7 @@ main( int argc, char **argv )
 			break;
 
 		case 'L':		/* the number of outerloops per client */
-			outerloops = strdup( optarg );
+			outerloops = optarg;
 			break;
 
 		case 'N':
@@ -325,19 +327,19 @@ main( int argc, char **argv )
 			break;
 
 		case 'P':		/* prog directory */
-			progdir = strdup( optarg );
-			break;
-
-		case 'p':		/* the servers port number */
-			port = strdup( optarg );
+			progdir = optarg;
 			break;
 
 		case 'r':		/* the number of retries in case of error */
-			retries = strdup( optarg );
+			retries = optarg;
+			break;
+
+		case 'S':
+			swamp++;
 			break;
 
 		case 't':		/* the delay in seconds between each retry */
-			delay = strdup( optarg );
+			delay = optarg;
 			break;
 
 		case 'w':		/* the managers passwd */
@@ -359,7 +361,7 @@ main( int argc, char **argv )
 		}
 	}
 
-	if (( dirname == NULL ) || ( port == NULL && uri == NULL ) ||
+	if (( dirname == NULL ) || ( uri == NULL ) ||
 			( manager == NULL ) || ( passwd == NULL ) || ( progdir == NULL ))
 	{
 		usage( argv[0], '\0' );
@@ -404,6 +406,9 @@ main( int argc, char **argv )
 
 	if ( pw_ask ) {
 		passwd = getpassphrase( _("Enter LDAP Password: ") );
+		if ( passwd == NULL ) { /* Allow EOF to exit. */
+			exit( EXIT_FAILURE );
+		}
 
 	} else if ( pw_file ) {
 		struct berval	pw;
@@ -476,7 +481,6 @@ main( int argc, char **argv )
 	}
 
 	/* setup friendly option */
-
 	switch ( friendly ) {
 	case 0:
 		break;
@@ -492,6 +496,15 @@ main( int argc, char **argv )
 		break;
 	}
 
+	/* setup swamp option */
+	if ( swamp ) {
+		swampopt[0] = '-';
+		if ( swamp > 3 ) swamp = 3;
+		swampopt[swamp + 1] = '\0';
+		for ( ; swamp-- > 0; ) swampopt[swamp + 1] = 'S';
+	}
+
+	/* setup loop options */
 	if ( sloops[0] == '\0' ) snprintf( sloops, sizeof( sloops ), "%d", 10 * loops );
 	if ( rloops[0] == '\0' ) snprintf( rloops, sizeof( rloops ), "%d", 20 * loops );
 	if ( aloops[0] == '\0' ) snprintf( aloops, sizeof( aloops ), "%d", loops );
@@ -507,15 +520,8 @@ main( int argc, char **argv )
 	snprintf( scmd, sizeof scmd, "%s" LDAP_DIRSEP SEARCHCMD,
 		progdir );
 	sargs[sanum++] = scmd;
-	if ( uri ) {
-		sargs[sanum++] = "-H";
-		sargs[sanum++] = uri;
-	} else {
-		sargs[sanum++] = "-h";
-		sargs[sanum++] = host;
-		sargs[sanum++] = "-p";
-		sargs[sanum++] = port;
-	}
+	sargs[sanum++] = "-H";
+	sargs[sanum++] = uri;
 	sargs[sanum++] = "-D";
 	sargs[sanum++] = manager;
 	sargs[sanum++] = "-w";
@@ -544,6 +550,9 @@ main( int argc, char **argv )
 		sargs[sanum++] = "-i";
 		sargs[sanum++] = ignore;
 	}
+	if ( swamp ) {
+		sargs[sanum++] = swampopt;
+	}
 	sargs[sanum++] = "-b";
 	sargs[sanum++] = NULL;		/* will hold the search base */
 	sargs[sanum++] = "-s";
@@ -565,15 +574,8 @@ main( int argc, char **argv )
 	snprintf( rcmd, sizeof rcmd, "%s" LDAP_DIRSEP READCMD,
 		progdir );
 	rargs[ranum++] = rcmd;
-	if ( uri ) {
-		rargs[ranum++] = "-H";
-		rargs[ranum++] = uri;
-	} else {
-		rargs[ranum++] = "-h";
-		rargs[ranum++] = host;
-		rargs[ranum++] = "-p";
-		rargs[ranum++] = port;
-	}
+	rargs[ranum++] = "-H";
+	rargs[ranum++] = uri;
 	rargs[ranum++] = "-D";
 	rargs[ranum++] = manager;
 	rargs[ranum++] = "-w";
@@ -599,6 +601,9 @@ main( int argc, char **argv )
 		rargs[ranum++] = "-i";
 		rargs[ranum++] = ignore;
 	}
+	if ( swamp ) {
+		rargs[ranum++] = swampopt;
+	}
 	rargs[ranum++] = "-e";
 	rargs[ranum++] = NULL;		/* will hold the read entry */
 
@@ -616,15 +621,8 @@ main( int argc, char **argv )
 	snprintf( ncmd, sizeof ncmd, "%s" LDAP_DIRSEP MODRDNCMD,
 		progdir );
 	nargs[nanum++] = ncmd;
-	if ( uri ) {
-		nargs[nanum++] = "-H";
-		nargs[nanum++] = uri;
-	} else {
-		nargs[nanum++] = "-h";
-		nargs[nanum++] = host;
-		nargs[nanum++] = "-p";
-		nargs[nanum++] = port;
-	}
+	nargs[nanum++] = "-H";
+	nargs[nanum++] = uri;
 	nargs[nanum++] = "-D";
 	nargs[nanum++] = manager;
 	nargs[nanum++] = "-w";
@@ -659,15 +657,8 @@ main( int argc, char **argv )
 	snprintf( mcmd, sizeof mcmd, "%s" LDAP_DIRSEP MODIFYCMD,
 		progdir );
 	margs[manum++] = mcmd;
-	if ( uri ) {
-		margs[manum++] = "-H";
-		margs[manum++] = uri;
-	} else {
-		margs[manum++] = "-h";
-		margs[manum++] = host;
-		margs[manum++] = "-p";
-		margs[manum++] = port;
-	}
+	margs[manum++] = "-H";
+	margs[manum++] = uri;
 	margs[manum++] = "-D";
 	margs[manum++] = manager;
 	margs[manum++] = "-w";
@@ -704,15 +695,8 @@ main( int argc, char **argv )
 	snprintf( acmd, sizeof acmd, "%s" LDAP_DIRSEP ADDCMD,
 		progdir );
 	aargs[aanum++] = acmd;
-	if ( uri ) {
-		aargs[aanum++] = "-H";
-		aargs[aanum++] = uri;
-	} else {
-		aargs[aanum++] = "-h";
-		aargs[aanum++] = host;
-		aargs[aanum++] = "-p";
-		aargs[aanum++] = port;
-	}
+	aargs[aanum++] = "-H";
+	aargs[aanum++] = uri;
 	aargs[aanum++] = "-D";
 	aargs[aanum++] = manager;
 	aargs[aanum++] = "-w";
@@ -750,25 +734,16 @@ main( int argc, char **argv )
 	if ( !noinit ) {
 		bargs[banum++] = "-I";	/* init on each bind */
 	}
-	if ( uri ) {
-		bargs[banum++] = "-H";
-		bargs[banum++] = uri;
-	} else {
-		bargs[banum++] = "-h";
-		bargs[banum++] = host;
-		bargs[banum++] = "-p";
-		bargs[banum++] = port;
-	}
+	bargs[banum++] = "-H";
+	bargs[banum++] = uri;
 	bargs[banum++] = "-l";
 	bargs[banum++] = bloops;
 	bargs[banum++] = "-L";
 	bargs[banum++] = outerloops;
-#if 0
 	bargs[banum++] = "-r";
 	bargs[banum++] = retries;
 	bargs[banum++] = "-t";
 	bargs[banum++] = delay;
-#endif
 	if ( friendly ) {
 		bargs[banum++] = friendlyOpt;
 	}
@@ -970,7 +945,8 @@ get_search_filters( char *filename, char *filters[], char *attrs[], char *bases[
 			} else {
 				bases[filter] = ArgDup( line );
 			}
-			fgets( line, BUFSIZ, fp );
+			if ( fgets( line, BUFSIZ, fp ) == NULL )
+				*line = '\0';
 			if (( nl = strchr( line, '\r' )) || ( nl = strchr( line, '\n' )))
 				*nl = '\0';
 
@@ -1039,6 +1015,9 @@ get_read_entries( char *filename, char *entries[], char *filters[] )
 				ldap_free_urldesc( lud );
 
 			} else {
+				if ( filters != NULL )
+					filters[entry] = NULL;
+
 				entries[entry] = ArgDup( line );
 			}
 
@@ -1104,17 +1083,17 @@ wait4kids( int nkidval )
 	int		status;
 
 	while ( nkids >= nkidval ) {
-		wait( &status );
+		pid_t pid = wait( &status );
 
 		if ( WIFSTOPPED(status) ) {
 			fprintf( stderr,
-			    "stopping: child stopped with signal %d\n",
-			    (int) WSTOPSIG(status) );
+			    "stopping: child PID=%ld stopped with signal %d\n",
+			    (long) pid, (int) WSTOPSIG(status) );
 
 		} else if ( WIFSIGNALED(status) ) {
 			fprintf( stderr, 
-			    "stopping: child terminated with signal %d%s\n",
-			    (int) WTERMSIG(status),
+			    "stopping: child PID=%ld terminated with signal %d%s\n",
+			    (long) pid, (int) WTERMSIG(status),
 #ifdef WCOREDUMP
 				WCOREDUMP(status) ? ", core dumped" : ""
 #else
@@ -1125,8 +1104,8 @@ wait4kids( int nkidval )
 
 		} else if ( WEXITSTATUS(status) != 0 ) {
 			fprintf( stderr, 
-			    "stopping: child exited with status %d\n",
-			    (int) WEXITSTATUS(status) );
+			    "stopping: child PID=%ld exited with status %d\n",
+			    (long) pid, (int) WEXITSTATUS(status) );
 			exit( WEXITSTATUS(status) );
 
 		} else {
